// ======================================================================
// 
// DirectDraw.h
// jeff grills
//
// copyright 1997 origin systems inc
//
// ======================================================================

#ifndef DIRECTDRAW_H
#define DIRECTDRAW_H

// ======================================================================

class Palette;
struct RGB;
#include <ddraw.h>

// ======================================================================

class DirectDraw
{
	friend class Direct3d;

public:

	struct Driver
	{
		int            id;
		char          *desc;
		char          *name;
		LPGUID         pGuid;
		GUID           guid;
		Driver        *next;
	};

	struct Mode
	{
		u_int8        *buffer;
		DDSURFACEDESC *sd;
		Mode          *next;
	};

private:

	static HWND                 windowHandle;
	static HWND                 drawWindowHandle;
	static LPDIRECTDRAW2        ddraw;
 	static LPDIRECTDRAWSURFACE  frontSurface;
	static LPDIRECTDRAWSURFACE  backSurface;
	static LPDIRECTDRAWSURFACE  lockSurface;
	static LPDIRECTDRAWCLIPPER  clipper;
	static DWORD                totalVideoMemory;

	static int                  width;
	static int                  height;
	static int                  depthBits;

	static int                  wantPages;
	static int                  havePages;
	static int                  visiblePage;
	static int                  alwaysBlit;

	static bool                 oneMegVideoCard;
	static bool                 bankSwitched;
	static bool                 backBufferInSystemRam;

	static int                  rMask;
	static int                  gMask;
	static int                  bMask;
	static u_int8               rSizeShift;
	static u_int8               gSizeShift;
	static u_int8               bSizeShift;
	static u_int8               rPlaceShift;
	static u_int8               gPlaceShift;
	static u_int8               bPlaceShift;

	static void                *buffer;
	static int                  pitch;
	static int                  lockCount;

	static Mode                *firstMode;
	static Mode                *currentMode;

	static int                  globalDriverId;
	static Driver              *firstDriver;
	static Driver              *lastDriver;
	static const Driver        *driver;
		
	static bool                 mfc;
	static bool                 windowed;

private:
	
	static void           computeShifts(int mask, u_int8 *placeShift, u_int8 *sizeShift);
	static BOOL WINAPI    directDrawEnumerateCallback(GUID FAR * lpGUID, LPSTR lpDriverDescription, LPSTR lpDriverName, LPVOID lpContext);
	static HRESULT WINAPI enumDisplayModesCallback(LPDDSURFACEDESC sd, LPVOID context);
	static void           setDisplayMode(int width, int height, int depthBits, bool anyWindowSize);

public:

	static void         enumerate(void);
	static void         install(HWND newWindowHandle, const Driver *newDriver, bool mfc);
	static void         remove(void);

	static const Driver *getDriver(int id);

	static void         getVideoMem(DWORD caps, DWORD *total, DWORD *free);

	static bool       isOneMegVideoCard(void);
	static bool       isBackBufferInSystemRam(void);
	static bool       isBankSwitched(void);

	// create the primary and back surfaces
	static void       doubleBuffer(void);
	static void       tripleBuffer(void);
	static void       createSurfaces(int width, int height, int depth);

	// access to variables for direct3d
	static LPDIRECTDRAW2        getDirectDrawObject(void);
	static LPDIRECTDRAWSURFACE  getBackSurface(void);

	// creation stuff
	static void       createPalette(LPDIRECTDRAWPALETTE *ddpal, const RGB *pal, int entries=256);
	static bool       createSurface(LPDIRECTDRAWSURFACE *surf, DDSURFACEDESC *desc, LPDIRECTDRAWPALETTE pal=NULL, bool allowFail=false);

	// frame buffer functions
	static void       setDrawWindow(HWND hwnd);
	static void       slam(void);
	static bool       clear(u_int32 background=0);
	static bool       isGdiVisible(void);

	// frame buffer access
	static int        screenWidth(void);
	static int        screenHeight(void);
	static int        screenDepthBits(void);
	static int        screenLock(bool backBuffer=true);
	static void       screenUnlock(void);
	static bool       screenIsLocked(void);
	static int        screenPitch(void);
	static void      *screenPtr(void);

	// color stuff
	static int        getRBitCount(void);
	static int        getGBitCount(void);
	static int        getBBitCount(void);
	static u_int8     getRSizeShift(void);
	static u_int8     getGSizeShift(void);
	static u_int8     getBSizeShift(void);
	static u_int8     getRPlaceShift(void);
	static u_int8     getGPlaceShift(void);
	static u_int8     getBPlaceShift(void);

	static u_int32    makeLFB(int r, int g, int b);
	static Pixel32    make888(u_int32 c);
};

// ======================================================================

inline int DirectDraw::screenWidth(void)
{
	return width;
}

inline int DirectDraw::screenHeight(void)
{
	return height;
}

inline int DirectDraw::screenDepthBits(void)
{
	return depthBits;
}

inline int DirectDraw::screenPitch(void)
{
	return pitch;
}

inline void *DirectDraw::screenPtr(void)
{
	return buffer;
}

inline bool DirectDraw::screenIsLocked(void)
{
	return lockCount != 0;
}

inline bool DirectDraw::isGdiVisible(void)
{
	return (visiblePage == 0);
}

inline void DirectDraw::doubleBuffer(void)
{
	wantPages = 1;
}

inline void DirectDraw::tripleBuffer(void)
{
	wantPages = 2;
}

inline LPDIRECTDRAW2 DirectDraw::getDirectDrawObject(void)
{
	return ddraw;
}

inline LPDIRECTDRAWSURFACE DirectDraw::getBackSurface(void)
{
	return backSurface;
}

inline int DirectDraw::getRBitCount(void)
{
	return 8 - rSizeShift;
}

inline int DirectDraw::getGBitCount(void)
{
	return 8 - gSizeShift;
}

inline int DirectDraw::getBBitCount(void)
{
	return 8 - bSizeShift;
}

inline u_int8 DirectDraw::getRSizeShift(void)
{
	return rSizeShift;
}

inline u_int8 DirectDraw::getGSizeShift(void)
{
	return gSizeShift;
}

inline u_int8 DirectDraw::getBSizeShift(void)
{
	return bSizeShift;
}

inline u_int8 DirectDraw::getRPlaceShift(void)
{
	return rPlaceShift;
}

inline u_int8 DirectDraw::getGPlaceShift(void)
{
	return gPlaceShift;
}

inline u_int8 DirectDraw::getBPlaceShift(void)
{
	return bPlaceShift;
}

inline u_int32 DirectDraw::makeLFB(int r, int g, int b)
{
	return
		(((r >> rSizeShift) << rPlaceShift) |
		 ((g >> gSizeShift) << gPlaceShift) |
		 ((b >> bSizeShift) << bPlaceShift));
}

inline Pixel32 DirectDraw::make888(u_int32 c)
{
	return
		((((c >> rPlaceShift) << rSizeShift) & 0xff) << 16) |
		((((c >> gPlaceShift) << gSizeShift) & 0xff) <<  8) |
		((((c >> bPlaceShift) << bSizeShift) & 0xff) <<  0);
}

inline bool DirectDraw::isOneMegVideoCard(void)
{
	return oneMegVideoCard;
}

inline bool DirectDraw::isBackBufferInSystemRam(void)
{
	return backBufferInSystemRam;
}

inline bool DirectDraw::isBankSwitched(void)
{
	return bankSwitched;
}

// ======================================================================

#endif
